// Code generated by 'ccgo -DSQLITE_PRIVATE= -export-defines  -export-enums  -export-externs X -export-fields F -export-typedefs  -ignore-unsupported-alignment -pkgname sqlite3 -o lib/sqlite_linux_amd64.go -trace-translation-units testdata/sqlite-amalgamation-3390200/sqlite3.c -full-path-comments -DNDEBUG -DHAVE_USLEEP -DLONGDOUBLE_TYPE=double -DSQLITE_CORE -DSQLITE_ENABLE_COLUMN_METADATA -DSQLITE_ENABLE_FTS5 -DSQLITE_ENABLE_GEOPOLY -DSQLITE_ENABLE_MATH_FUNCTIONS -DSQLITE_ENABLE_MEMORY_MANAGEMENT -DSQLITE_ENABLE_OFFSET_SQL_FUNC -DSQLITE_ENABLE_PREUPDATE_HOOK -DSQLITE_ENABLE_RBU -DSQLITE_ENABLE_RTREE -DSQLITE_ENABLE_SESSION -DSQLITE_ENABLE_SNAPSHOT -DSQLITE_ENABLE_STAT4 -DSQLITE_ENABLE_UNLOCK_NOTIFY -DSQLITE_LIKE_DOESNT_MATCH_BLOBS -DSQLITE_MUTEX_APPDEF=1 -DSQLITE_MUTEX_NOOP -DSQLITE_SOUNDEX -DSQLITE_THREADSAFE=1 -DSQLITE_OS_UNIX=1', DO NOT EDIT.

package sqlite3

var CAPI = map[string]struct{}{
	"sqlite3AbsInt32":                              {},
	"sqlite3AddCheckConstraint":                    {},
	"sqlite3AddCollateType":                        {},
	"sqlite3AddColumn":                             {},
	"sqlite3AddDefaultValue":                       {},
	"sqlite3AddGenerated":                          {},
	"sqlite3AddInt64":                              {},
	"sqlite3AddNotNull":                            {},
	"sqlite3AddPrimaryKey":                         {},
	"sqlite3AddReturning":                          {},
	"sqlite3AffinityType":                          {},
	"sqlite3AggInfoPersistWalkerInit":              {},
	"sqlite3AllocateIndexObject":                   {},
	"sqlite3AlterBeginAddColumn":                   {},
	"sqlite3AlterDropColumn":                       {},
	"sqlite3AlterFinishAddColumn":                  {},
	"sqlite3AlterFunctions":                        {},
	"sqlite3AlterRenameColumn":                     {},
	"sqlite3AlterRenameTable":                      {},
	"sqlite3AnalysisLoad":                          {},
	"sqlite3Analyze":                               {},
	"sqlite3ApiExit":                               {},
	"sqlite3ArrayAllocate":                         {},
	"sqlite3AtoF":                                  {},
	"sqlite3Atoi":                                  {},
	"sqlite3Atoi64":                                {},
	"sqlite3Attach":                                {},
	"sqlite3AuthCheck":                             {},
	"sqlite3AuthContextPop":                        {},
	"sqlite3AuthContextPush":                       {},
	"sqlite3AuthRead":                              {},
	"sqlite3AuthReadCol":                           {},
	"sqlite3AutoLoadExtensions":                    {},
	"sqlite3AutoincrementBegin":                    {},
	"sqlite3AutoincrementEnd":                      {},
	"sqlite3BackupRestart":                         {},
	"sqlite3BackupUpdate":                          {},
	"sqlite3BeginBenignMalloc":                     {},
	"sqlite3BeginTransaction":                      {},
	"sqlite3BeginTrigger":                          {},
	"sqlite3BeginWriteOperation":                   {},
	"sqlite3BenignMallocHooks":                     {},
	"sqlite3BinaryCompareCollSeq":                  {},
	"sqlite3BitvecBuiltinTest":                     {},
	"sqlite3BitvecClear":                           {},
	"sqlite3BitvecCreate":                          {},
	"sqlite3BitvecDestroy":                         {},
	"sqlite3BitvecSet":                             {},
	"sqlite3BitvecSize":                            {},
	"sqlite3BitvecTest":                            {},
	"sqlite3BitvecTestNotNull":                     {},
	"sqlite3BlobCompare":                           {},
	"sqlite3BtreeBeginStmt":                        {},
	"sqlite3BtreeBeginTrans":                       {},
	"sqlite3BtreeCheckpoint":                       {},
	"sqlite3BtreeClearCursor":                      {},
	"sqlite3BtreeClearTable":                       {},
	"sqlite3BtreeClearTableOfCursor":               {},
	"sqlite3BtreeClose":                            {},
	"sqlite3BtreeCloseCursor":                      {},
	"sqlite3BtreeCommit":                           {},
	"sqlite3BtreeCommitPhaseOne":                   {},
	"sqlite3BtreeCommitPhaseTwo":                   {},
	"sqlite3BtreeConnectionCount":                  {},
	"sqlite3BtreeCopyFile":                         {},
	"sqlite3BtreeCount":                            {},
	"sqlite3BtreeCreateTable":                      {},
	"sqlite3BtreeCursor":                           {},
	"sqlite3BtreeCursorHasHint":                    {},
	"sqlite3BtreeCursorHasMoved":                   {},
	"sqlite3BtreeCursorHintFlags":                  {},
	"sqlite3BtreeCursorIsValidNN":                  {},
	"sqlite3BtreeCursorPin":                        {},
	"sqlite3BtreeCursorRestore":                    {},
	"sqlite3BtreeCursorSize":                       {},
	"sqlite3BtreeCursorUnpin":                      {},
	"sqlite3BtreeCursorZero":                       {},
	"sqlite3BtreeDelete":                           {},
	"sqlite3BtreeDropTable":                        {},
	"sqlite3BtreeEnter":                            {},
	"sqlite3BtreeEnterAll":                         {},
	"sqlite3BtreeEnterCursor":                      {},
	"sqlite3BtreeEof":                              {},
	"sqlite3BtreeFakeValidCursor":                  {},
	"sqlite3BtreeFirst":                            {},
	"sqlite3BtreeGetAutoVacuum":                    {},
	"sqlite3BtreeGetFilename":                      {},
	"sqlite3BtreeGetJournalname":                   {},
	"sqlite3BtreeGetMeta":                          {},
	"sqlite3BtreeGetPageSize":                      {},
	"sqlite3BtreeGetRequestedReserve":              {},
	"sqlite3BtreeGetReserveNoMutex":                {},
	"sqlite3BtreeIncrVacuum":                       {},
	"sqlite3BtreeIncrblobCursor":                   {},
	"sqlite3BtreeIndexMoveto":                      {},
	"sqlite3BtreeInsert":                           {},
	"sqlite3BtreeIntegerKey":                       {},
	"sqlite3BtreeIntegrityCheck":                   {},
	"sqlite3BtreeIsInBackup":                       {},
	"sqlite3BtreeIsReadonly":                       {},
	"sqlite3BtreeLast":                             {},
	"sqlite3BtreeLastPage":                         {},
	"sqlite3BtreeLeave":                            {},
	"sqlite3BtreeLeaveAll":                         {},
	"sqlite3BtreeLeaveCursor":                      {},
	"sqlite3BtreeLockTable":                        {},
	"sqlite3BtreeMaxPageCount":                     {},
	"sqlite3BtreeMaxRecordSize":                    {},
	"sqlite3BtreeNewDb":                            {},
	"sqlite3BtreeNext":                             {},
	"sqlite3BtreeOffset":                           {},
	"sqlite3BtreeOpen":                             {},
	"sqlite3BtreePager":                            {},
	"sqlite3BtreePayload":                          {},
	"sqlite3BtreePayloadChecked":                   {},
	"sqlite3BtreePayloadFetch":                     {},
	"sqlite3BtreePayloadSize":                      {},
	"sqlite3BtreePrevious":                         {},
	"sqlite3BtreePutData":                          {},
	"sqlite3BtreeRollback":                         {},
	"sqlite3BtreeRowCountEst":                      {},
	"sqlite3BtreeSavepoint":                        {},
	"sqlite3BtreeSchema":                           {},
	"sqlite3BtreeSchemaLocked":                     {},
	"sqlite3BtreeSecureDelete":                     {},
	"sqlite3BtreeSetAutoVacuum":                    {},
	"sqlite3BtreeSetCacheSize":                     {},
	"sqlite3BtreeSetMmapLimit":                     {},
	"sqlite3BtreeSetPageSize":                      {},
	"sqlite3BtreeSetPagerFlags":                    {},
	"sqlite3BtreeSetSpillSize":                     {},
	"sqlite3BtreeSetVersion":                       {},
	"sqlite3BtreeSharable":                         {},
	"sqlite3BtreeTableMoveto":                      {},
	"sqlite3BtreeTransferRow":                      {},
	"sqlite3BtreeTripAllCursors":                   {},
	"sqlite3BtreeTxnState":                         {},
	"sqlite3BtreeUpdateMeta":                       {},
	"sqlite3BuiltinFunctions":                      {},
	"sqlite3CantopenError":                         {},
	"sqlite3ChangeCookie":                          {},
	"sqlite3CheckCollSeq":                          {},
	"sqlite3CheckObjectName":                       {},
	"sqlite3Checkpoint":                            {},
	"sqlite3ClearOnOrUsing":                        {},
	"sqlite3ClearTempRegCache":                     {},
	"sqlite3CloseExtensions":                       {},
	"sqlite3CloseSavepoints":                       {},
	"sqlite3CodeChangeCount":                       {},
	"sqlite3CodeDropTable":                         {},
	"sqlite3CodeRhsOfIN":                           {},
	"sqlite3CodeRowTrigger":                        {},
	"sqlite3CodeRowTriggerDirect":                  {},
	"sqlite3CodeSubselect":                         {},
	"sqlite3CodeVerifyNamedSchema":                 {},
	"sqlite3CodeVerifySchema":                      {},
	"sqlite3CollapseDatabaseArray":                 {},
	"sqlite3ColumnColl":                            {},
	"sqlite3ColumnDefault":                         {},
	"sqlite3ColumnExpr":                            {},
	"sqlite3ColumnIndex":                           {},
	"sqlite3ColumnSetColl":                         {},
	"sqlite3ColumnSetExpr":                         {},
	"sqlite3ColumnType":                            {},
	"sqlite3ColumnsFromExprList":                   {},
	"sqlite3CommitInternalChanges":                 {},
	"sqlite3CompareAffinity":                       {},
	"sqlite3CompileOptions":                        {},
	"sqlite3CompleteInsertion":                     {},
	"sqlite3ComputeGeneratedColumns":               {},
	"sqlite3Config":                                {},
	"sqlite3ConnectionBlocked":                     {},
	"sqlite3ConnectionClosed":                      {},
	"sqlite3ConnectionUnlocked":                    {},
	"sqlite3CorruptError":                          {},
	"sqlite3CreateColumnExpr":                      {},
	"sqlite3CreateForeignKey":                      {},
	"sqlite3CreateFunc":                            {},
	"sqlite3CreateIndex":                           {},
	"sqlite3CreateView":                            {},
	"sqlite3CteDelete":                             {},
	"sqlite3CteNew":                                {},
	"sqlite3CtypeMap":                              {},
	"sqlite3DbFree":                                {},
	"sqlite3DbFreeNN":                              {},
	"sqlite3DbIsNamed":                             {},
	"sqlite3DbMallocRaw":                           {},
	"sqlite3DbMallocRawNN":                         {},
	"sqlite3DbMallocSize":                          {},
	"sqlite3DbMallocZero":                          {},
	"sqlite3DbNameToBtree":                         {},
	"sqlite3DbRealloc":                             {},
	"sqlite3DbReallocOrFree":                       {},
	"sqlite3DbSpanDup":                             {},
	"sqlite3DbStrDup":                              {},
	"sqlite3DbStrNDup":                             {},
	"sqlite3DecOrHexToI64":                         {},
	"sqlite3DefaultMutex":                          {},
	"sqlite3DefaultRowEst":                         {},
	"sqlite3DeferForeignKey":                       {},
	"sqlite3DeleteColumnNames":                     {},
	"sqlite3DeleteFrom":                            {},
	"sqlite3DeleteIndexSamples":                    {},
	"sqlite3DeleteTable":                           {},
	"sqlite3DeleteTrigger":                         {},
	"sqlite3DeleteTriggerStep":                     {},
	"sqlite3Dequote":                               {},
	"sqlite3DequoteExpr":                           {},
	"sqlite3DequoteToken":                          {},
	"sqlite3Detach":                                {},
	"sqlite3DropIndex":                             {},
	"sqlite3DropTable":                             {},
	"sqlite3DropTrigger":                           {},
	"sqlite3DropTriggerPtr":                        {},
	"sqlite3EndBenignMalloc":                       {},
	"sqlite3EndTable":                              {},
	"sqlite3EndTransaction":                        {},
	"sqlite3ErrStr":                                {},
	"sqlite3Error":                                 {},
	"sqlite3ErrorClear":                            {},
	"sqlite3ErrorMsg":                              {},
	"sqlite3ErrorToParser":                         {},
	"sqlite3ErrorWithMsg":                          {},
	"sqlite3ExpandSubquery":                        {},
	"sqlite3ExpirePreparedStatements":              {},
	"sqlite3Expr":                                  {},
	"sqlite3ExprAddCollateString":                  {},
	"sqlite3ExprAddCollateToken":                   {},
	"sqlite3ExprAffinity":                          {},
	"sqlite3ExprAlloc":                             {},
	"sqlite3ExprAnalyzeAggList":                    {},
	"sqlite3ExprAnalyzeAggregates":                 {},
	"sqlite3ExprAnd":                               {},
	"sqlite3ExprAssignVarNumber":                   {},
	"sqlite3ExprAttachSubtrees":                    {},
	"sqlite3ExprCanBeNull":                         {},
	"sqlite3ExprCheckHeight":                       {},
	"sqlite3ExprCheckIN":                           {},
	"sqlite3ExprCode":                              {},
	"sqlite3ExprCodeCopy":                          {},
	"sqlite3ExprCodeExprList":                      {},
	"sqlite3ExprCodeFactorable":                    {},
	"sqlite3ExprCodeGeneratedColumn":               {},
	"sqlite3ExprCodeGetColumn":                     {},
	"sqlite3ExprCodeGetColumnOfTable":              {},
	"sqlite3ExprCodeLoadIndexColumn":               {},
	"sqlite3ExprCodeMove":                          {},
	"sqlite3ExprCodeRunJustOnce":                   {},
	"sqlite3ExprCodeTarget":                        {},
	"sqlite3ExprCodeTemp":                          {},
	"sqlite3ExprColUsed":                           {},
	"sqlite3ExprCollSeq":                           {},
	"sqlite3ExprCollSeqMatch":                      {},
	"sqlite3ExprCompare":                           {},
	"sqlite3ExprCompareCollSeq":                    {},
	"sqlite3ExprCompareSkip":                       {},
	"sqlite3ExprCoveredByIndex":                    {},
	"sqlite3ExprDeferredDelete":                    {},
	"sqlite3ExprDelete":                            {},
	"sqlite3ExprDup":                               {},
	"sqlite3ExprForVectorField":                    {},
	"sqlite3ExprFunction":                          {},
	"sqlite3ExprFunctionUsable":                    {},
	"sqlite3ExprIdToTrueFalse":                     {},
	"sqlite3ExprIfFalse":                           {},
	"sqlite3ExprIfFalseDup":                        {},
	"sqlite3ExprIfTrue":                            {},
	"sqlite3ExprImpliesExpr":                       {},
	"sqlite3ExprImpliesNonNullRow":                 {},
	"sqlite3ExprIsConstant":                        {},
	"sqlite3ExprIsConstantNotJoin":                 {},
	"sqlite3ExprIsConstantOrFunction":              {},
	"sqlite3ExprIsConstantOrGroupBy":               {},
	"sqlite3ExprIsInteger":                         {},
	"sqlite3ExprIsTableConstant":                   {},
	"sqlite3ExprIsTableConstraint":                 {},
	"sqlite3ExprIsVector":                          {},
	"sqlite3ExprListAppend":                        {},
	"sqlite3ExprListAppendGrow":                    {},
	"sqlite3ExprListAppendNew":                     {},
	"sqlite3ExprListAppendVector":                  {},
	"sqlite3ExprListCheckLength":                   {},
	"sqlite3ExprListCompare":                       {},
	"sqlite3ExprListDelete":                        {},
	"sqlite3ExprListDup":                           {},
	"sqlite3ExprListFlags":                         {},
	"sqlite3ExprListSetName":                       {},
	"sqlite3ExprListSetSortOrder":                  {},
	"sqlite3ExprListSetSpan":                       {},
	"sqlite3ExprListToValues":                      {},
	"sqlite3ExprNNCollSeq":                         {},
	"sqlite3ExprNeedsNoAffinityChange":             {},
	"sqlite3ExprReferencesUpdatedColumn":           {},
	"sqlite3ExprSetHeightAndFlags":                 {},
	"sqlite3ExprSimplifiedAndOr":                   {},
	"sqlite3ExprSkipCollate":                       {},
	"sqlite3ExprSkipCollateAndLikely":              {},
	"sqlite3ExprTruthValue":                        {},
	"sqlite3ExprUnmapAndDelete":                    {},
	"sqlite3ExprVectorSize":                        {},
	"sqlite3ExprWalkNoop":                          {},
	"sqlite3FaultSim":                              {},
	"sqlite3FindCollSeq":                           {},
	"sqlite3FindDb":                                {},
	"sqlite3FindDbName":                            {},
	"sqlite3FindFunction":                          {},
	"sqlite3FindInIndex":                           {},
	"sqlite3FindIndex":                             {},
	"sqlite3FindTable":                             {},
	"sqlite3FinishCoding":                          {},
	"sqlite3FinishTrigger":                         {},
	"sqlite3FixExpr":                               {},
	"sqlite3FixInit":                               {},
	"sqlite3FixSelect":                             {},
	"sqlite3FixSrcList":                            {},
	"sqlite3FixTriggerStep":                        {},
	"sqlite3FkActions":                             {},
	"sqlite3FkCheck":                               {},
	"sqlite3FkClearTriggerCache":                   {},
	"sqlite3FkDelete":                              {},
	"sqlite3FkDropTable":                           {},
	"sqlite3FkLocateIndex":                         {},
	"sqlite3FkOldmask":                             {},
	"sqlite3FkReferences":                          {},
	"sqlite3FkRequired":                            {},
	"sqlite3FreeIndex":                             {},
	"sqlite3Fts5Init":                              {},
	"sqlite3FunctionSearch":                        {},
	"sqlite3GenerateColumnNames":                   {},
	"sqlite3GenerateConstraintChecks":              {},
	"sqlite3GenerateIndexKey":                      {},
	"sqlite3GenerateRowDelete":                     {},
	"sqlite3GenerateRowIndexDelete":                {},
	"sqlite3Get4byte":                              {},
	"sqlite3GetBoolean":                            {},
	"sqlite3GetCollSeq":                            {},
	"sqlite3GetInt32":                              {},
	"sqlite3GetTempRange":                          {},
	"sqlite3GetTempReg":                            {},
	"sqlite3GetToken":                              {},
	"sqlite3GetUInt32":                             {},
	"sqlite3GetVTable":                             {},
	"sqlite3GetVarint":                             {},
	"sqlite3GetVarint32":                           {},
	"sqlite3GetVdbe":                               {},
	"sqlite3HaltConstraint":                        {},
	"sqlite3HasExplicitNulls":                      {},
	"sqlite3HashClear":                             {},
	"sqlite3HashFind":                              {},
	"sqlite3HashInit":                              {},
	"sqlite3HashInsert":                            {},
	"sqlite3HeaderSizeBtree":                       {},
	"sqlite3HeaderSizePcache":                      {},
	"sqlite3HeaderSizePcache1":                     {},
	"sqlite3HeapNearlyFull":                        {},
	"sqlite3HexToBlob":                             {},
	"sqlite3HexToInt":                              {},
	"sqlite3IdListAppend":                          {},
	"sqlite3IdListDelete":                          {},
	"sqlite3IdListDup":                             {},
	"sqlite3IdListIndex":                           {},
	"sqlite3IndexAffinityOk":                       {},
	"sqlite3IndexAffinityStr":                      {},
	"sqlite3IndexColumnAffinity":                   {},
	"sqlite3IndexHasDuplicateRootPage":             {},
	"sqlite3IndexedByLookup":                       {},
	"sqlite3Init":                                  {},
	"sqlite3InitCallback":                          {},
	"sqlite3InitOne":                               {},
	"sqlite3Insert":                                {},
	"sqlite3InsertBuiltinFuncs":                    {},
	"sqlite3Int64ToText":                           {},
	"sqlite3IntFloatCompare":                       {},
	"sqlite3InvokeBusyHandler":                     {},
	"sqlite3IsBinary":                              {},
	"sqlite3IsIdChar":                              {},
	"sqlite3IsLikeFunction":                        {},
	"sqlite3IsNaN":                                 {},
	"sqlite3IsReadOnly":                            {},
	"sqlite3IsRowid":                               {},
	"sqlite3IsShadowTableOf":                       {},
	"sqlite3IsTrueOrFalse":                         {},
	"sqlite3JoinType":                              {},
	"sqlite3JournalIsInMemory":                     {},
	"sqlite3JournalModename":                       {},
	"sqlite3JournalOpen":                           {},
	"sqlite3JournalSize":                           {},
	"sqlite3JsonTableFunctions":                    {},
	"sqlite3KeyInfoAlloc":                          {},
	"sqlite3KeyInfoFromExprList":                   {},
	"sqlite3KeyInfoOfIndex":                        {},
	"sqlite3KeyInfoRef":                            {},
	"sqlite3KeyInfoUnref":                          {},
	"sqlite3KeywordCode":                           {},
	"sqlite3LeaveMutexAndCloseZombie":              {},
	"sqlite3LocateCollSeq":                         {},
	"sqlite3LocateTable":                           {},
	"sqlite3LocateTableItem":                       {},
	"sqlite3LogEst":                                {},
	"sqlite3LogEstAdd":                             {},
	"sqlite3LogEstFromDouble":                      {},
	"sqlite3LogEstToInt":                           {},
	"sqlite3LookasideUsed":                         {},
	"sqlite3MPrintf":                               {},
	"sqlite3Malloc":                                {},
	"sqlite3MallocEnd":                             {},
	"sqlite3MallocInit":                            {},
	"sqlite3MallocMutex":                           {},
	"sqlite3MallocSize":                            {},
	"sqlite3MallocZero":                            {},
	"sqlite3MarkAllShadowTablesOf":                 {},
	"sqlite3MatchEName":                            {},
	"sqlite3MaterializeView":                       {},
	"sqlite3MayAbort":                              {},
	"sqlite3MemCompare":                            {},
	"sqlite3MemJournalOpen":                        {},
	"sqlite3MemSetDefault":                         {},
	"sqlite3MemdbInit":                             {},
	"sqlite3MisuseError":                           {},
	"sqlite3MulInt64":                              {},
	"sqlite3MultiWrite":                            {},
	"sqlite3MutexAlloc":                            {},
	"sqlite3MutexEnd":                              {},
	"sqlite3MutexInit":                             {},
	"sqlite3NameFromToken":                         {},
	"sqlite3NestedParse":                           {},
	"sqlite3NoopDestructor":                        {},
	"sqlite3NoopMutex":                             {},
	"sqlite3NotPureFunc":                           {},
	"sqlite3OomClear":                              {},
	"sqlite3OomFault":                              {},
	"sqlite3OpcodeName":                            {},
	"sqlite3OpcodeProperty":                        {},
	"sqlite3OpenSchemaTable":                       {},
	"sqlite3OpenTable":                             {},
	"sqlite3OpenTableAndIndices":                   {},
	"sqlite3OpenTempDatabase":                      {},
	"sqlite3OsAccess":                              {},
	"sqlite3OsCheckReservedLock":                   {},
	"sqlite3OsClose":                               {},
	"sqlite3OsCloseFree":                           {},
	"sqlite3OsCurrentTimeInt64":                    {},
	"sqlite3OsDelete":                              {},
	"sqlite3OsDeviceCharacteristics":               {},
	"sqlite3OsDlClose":                             {},
	"sqlite3OsDlError":                             {},
	"sqlite3OsDlOpen":                              {},
	"sqlite3OsDlSym":                               {},
	"sqlite3OsFetch":                               {},
	"sqlite3OsFileControl":                         {},
	"sqlite3OsFileControlHint":                     {},
	"sqlite3OsFileSize":                            {},
	"sqlite3OsFullPathname":                        {},
	"sqlite3OsGetLastError":                        {},
	"sqlite3OsInit":                                {},
	"sqlite3OsLock":                                {},
	"sqlite3OsOpen":                                {},
	"sqlite3OsOpenMalloc":                          {},
	"sqlite3OsRandomness":                          {},
	"sqlite3OsRead":                                {},
	"sqlite3OsSectorSize":                          {},
	"sqlite3OsShmBarrier":                          {},
	"sqlite3OsShmLock":                             {},
	"sqlite3OsShmMap":                              {},
	"sqlite3OsShmUnmap":                            {},
	"sqlite3OsSleep":                               {},
	"sqlite3OsSync":                                {},
	"sqlite3OsTruncate":                            {},
	"sqlite3OsUnfetch":                             {},
	"sqlite3OsUnlock":                              {},
	"sqlite3OsWrite":                               {},
	"sqlite3PCacheBufferSetup":                     {},
	"sqlite3PCachePercentDirty":                    {},
	"sqlite3PCacheSetDefault":                      {},
	"sqlite3PExpr":                                 {},
	"sqlite3PExprAddSelect":                        {},
	"sqlite3PageFree":                              {},
	"sqlite3PageMalloc":                            {},
	"sqlite3PagerBackupPtr":                        {},
	"sqlite3PagerBegin":                            {},
	"sqlite3PagerCacheStat":                        {},
	"sqlite3PagerCheckpoint":                       {},
	"sqlite3PagerClearCache":                       {},
	"sqlite3PagerClose":                            {},
	"sqlite3PagerCloseWal":                         {},
	"sqlite3PagerCommitPhaseOne":                   {},
	"sqlite3PagerCommitPhaseTwo":                   {},
	"sqlite3PagerDataVersion":                      {},
	"sqlite3PagerDontWrite":                        {},
	"sqlite3PagerExclusiveLock":                    {},
	"sqlite3PagerFile":                             {},
	"sqlite3PagerFilename":                         {},
	"sqlite3PagerFlush":                            {},
	"sqlite3PagerGet":                              {},
	"sqlite3PagerGetData":                          {},
	"sqlite3PagerGetExtra":                         {},
	"sqlite3PagerGetJournalMode":                   {},
	"sqlite3PagerIsMemdb":                          {},
	"sqlite3PagerIsreadonly":                       {},
	"sqlite3PagerJournalSizeLimit":                 {},
	"sqlite3PagerJournalname":                      {},
	"sqlite3PagerJrnlFile":                         {},
	"sqlite3PagerLockingMode":                      {},
	"sqlite3PagerLookup":                           {},
	"sqlite3PagerMaxPageCount":                     {},
	"sqlite3PagerMemUsed":                          {},
	"sqlite3PagerMovepage":                         {},
	"sqlite3PagerOkToChangeJournalMode":            {},
	"sqlite3PagerOpen":                             {},
	"sqlite3PagerOpenSavepoint":                    {},
	"sqlite3PagerOpenWal":                          {},
	"sqlite3PagerPageRefcount":                     {},
	"sqlite3PagerPagecount":                        {},
	"sqlite3PagerReadFileheader":                   {},
	"sqlite3PagerRef":                              {},
	"sqlite3PagerRekey":                            {},
	"sqlite3PagerRollback":                         {},
	"sqlite3PagerSavepoint":                        {},
	"sqlite3PagerSetBusyHandler":                   {},
	"sqlite3PagerSetCachesize":                     {},
	"sqlite3PagerSetFlags":                         {},
	"sqlite3PagerSetJournalMode":                   {},
	"sqlite3PagerSetMmapLimit":                     {},
	"sqlite3PagerSetPagesize":                      {},
	"sqlite3PagerSetSpillsize":                     {},
	"sqlite3PagerSharedLock":                       {},
	"sqlite3PagerShrink":                           {},
	"sqlite3PagerSnapshotCheck":                    {},
	"sqlite3PagerSnapshotGet":                      {},
	"sqlite3PagerSnapshotOpen":                     {},
	"sqlite3PagerSnapshotRecover":                  {},
	"sqlite3PagerSnapshotUnlock":                   {},
	"sqlite3PagerSync":                             {},
	"sqlite3PagerTempSpace":                        {},
	"sqlite3PagerTruncateImage":                    {},
	"sqlite3PagerUnref":                            {},
	"sqlite3PagerUnrefNotNull":                     {},
	"sqlite3PagerUnrefPageOne":                     {},
	"sqlite3PagerVfs":                              {},
	"sqlite3PagerWalCallback":                      {},
	"sqlite3PagerWalSupported":                     {},
	"sqlite3PagerWrite":                            {},
	"sqlite3ParseObjectInit":                       {},
	"sqlite3ParseObjectReset":                      {},
	"sqlite3ParseUri":                              {},
	"sqlite3Parser":                                {},
	"sqlite3ParserAddCleanup":                      {},
	"sqlite3ParserFallback":                        {},
	"sqlite3ParserFinalize":                        {},
	"sqlite3ParserInit":                            {},
	"sqlite3Pcache1Mutex":                          {},
	"sqlite3PcacheCleanAll":                        {},
	"sqlite3PcacheClear":                           {},
	"sqlite3PcacheClearSyncFlags":                  {},
	"sqlite3PcacheClearWritable":                   {},
	"sqlite3PcacheClose":                           {},
	"sqlite3PcacheDirtyList":                       {},
	"sqlite3PcacheDrop":                            {},
	"sqlite3PcacheFetch":                           {},
	"sqlite3PcacheFetchFinish":                     {},
	"sqlite3PcacheFetchStress":                     {},
	"sqlite3PcacheInitialize":                      {},
	"sqlite3PcacheMakeClean":                       {},
	"sqlite3PcacheMakeDirty":                       {},
	"sqlite3PcacheMove":                            {},
	"sqlite3PcacheOpen":                            {},
	"sqlite3PcachePageRefcount":                    {},
	"sqlite3PcachePagecount":                       {},
	"sqlite3PcacheRef":                             {},
	"sqlite3PcacheRefCount":                        {},
	"sqlite3PcacheRelease":                         {},
	"sqlite3PcacheReleaseMemory":                   {},
	"sqlite3PcacheSetCachesize":                    {},
	"sqlite3PcacheSetPageSize":                     {},
	"sqlite3PcacheSetSpillsize":                    {},
	"sqlite3PcacheShrink":                          {},
	"sqlite3PcacheShutdown":                        {},
	"sqlite3PcacheSize":                            {},
	"sqlite3PcacheTruncate":                        {},
	"sqlite3PendingByte":                           {},
	"sqlite3Pragma":                                {},
	"sqlite3PragmaVtabRegister":                    {},
	"sqlite3PreferredTableName":                    {},
	"sqlite3PrimaryKeyIndex":                       {},
	"sqlite3PrngRestoreState":                      {},
	"sqlite3PrngSaveState":                         {},
	"sqlite3Put4byte":                              {},
	"sqlite3PutVarint":                             {},
	"sqlite3QuoteValue":                            {},
	"sqlite3ReadOnlyShadowTables":                  {},
	"sqlite3ReadSchema":                            {},
	"sqlite3RealSameAsInt":                         {},
	"sqlite3Realloc":                               {},
	"sqlite3RecordErrorByteOffset":                 {},
	"sqlite3RecordErrorOffsetOfExpr":               {},
	"sqlite3ReferencesSrcList":                     {},
	"sqlite3RegisterBuiltinFunctions":              {},
	"sqlite3RegisterDateTimeFunctions":             {},
	"sqlite3RegisterJsonFunctions":                 {},
	"sqlite3RegisterLikeFunctions":                 {},
	"sqlite3RegisterPerConnectionBuiltinFunctions": {},
	"sqlite3Reindex":                               {},
	"sqlite3ReleaseTempRange":                      {},
	"sqlite3ReleaseTempReg":                        {},
	"sqlite3RenameExprUnmap":                       {},
	"sqlite3RenameExprlistUnmap":                   {},
	"sqlite3RenameTokenMap":                        {},
	"sqlite3RenameTokenRemap":                      {},
	"sqlite3ReportError":                           {},
	"sqlite3Reprepare":                             {},
	"sqlite3ResetAllSchemasOfConnection":           {},
	"sqlite3ResetOneSchema":                        {},
	"sqlite3ResolveExprListNames":                  {},
	"sqlite3ResolveExprNames":                      {},
	"sqlite3ResolveOrderGroupBy":                   {},
	"sqlite3ResolvePartIdxLabel":                   {},
	"sqlite3ResolveSelectNames":                    {},
	"sqlite3ResolveSelfReference":                  {},
	"sqlite3ResultIntReal":                         {},
	"sqlite3ResultSetOfSelect":                     {},
	"sqlite3ResultStrAccum":                        {},
	"sqlite3RollbackAll":                           {},
	"sqlite3RootPageMoved":                         {},
	"sqlite3RowSetClear":                           {},
	"sqlite3RowSetDelete":                          {},
	"sqlite3RowSetInit":                            {},
	"sqlite3RowSetInsert":                          {},
	"sqlite3RowSetNext":                            {},
	"sqlite3RowSetTest":                            {},
	"sqlite3RowidConstraint":                       {},
	"sqlite3RtreeInit":                             {},
	"sqlite3RunParser":                             {},
	"sqlite3RunVacuum":                             {},
	"sqlite3SafetyCheckOk":                         {},
	"sqlite3SafetyCheckSickOrOk":                   {},
	"sqlite3Savepoint":                             {},
	"sqlite3SchemaClear":                           {},
	"sqlite3SchemaGet":                             {},
	"sqlite3SchemaToIndex":                         {},
	"sqlite3SectorSize":                            {},
	"sqlite3Select":                                {},
	"sqlite3SelectAddColumnTypeAndCollation":       {},
	"sqlite3SelectDelete":                          {},
	"sqlite3SelectDestInit":                        {},
	"sqlite3SelectDup":                             {},
	"sqlite3SelectExprHeight":                      {},
	"sqlite3SelectNew":                             {},
	"sqlite3SelectOpName":                          {},
	"sqlite3SelectPopWith":                         {},
	"sqlite3SelectPrep":                            {},
	"sqlite3SelectWalkFail":                        {},
	"sqlite3SelectWalkNoop":                        {},
	"sqlite3SelectWrongNumTermsError":              {},
	"sqlite3SetJoinExpr":                           {},
	"sqlite3SetString":                             {},
	"sqlite3SetTextEncoding":                       {},
	"sqlite3ShadowTableName":                       {},
	"sqlite3SmallTypeSizes":                        {},
	"sqlite3SrcItemColumnUsed":                     {},
	"sqlite3SrcListAppend":                         {},
	"sqlite3SrcListAppendFromTerm":                 {},
	"sqlite3SrcListAppendList":                     {},
	"sqlite3SrcListAssignCursors":                  {},
	"sqlite3SrcListDelete":                         {},
	"sqlite3SrcListDup":                            {},
	"sqlite3SrcListEnlarge":                        {},
	"sqlite3SrcListFuncArgs":                       {},
	"sqlite3SrcListIndexedBy":                      {},
	"sqlite3SrcListLookup":                         {},
	"sqlite3SrcListShiftJoinType":                  {},
	"sqlite3StartTable":                            {},
	"sqlite3Stat4Column":                           {},
	"sqlite3Stat4ProbeFree":                        {},
	"sqlite3Stat4ProbeSetValue":                    {},
	"sqlite3Stat4ValueFromExpr":                    {},
	"sqlite3StatusDown":                            {},
	"sqlite3StatusHighwater":                       {},
	"sqlite3StatusUp":                              {},
	"sqlite3StatusValue":                           {},
	"sqlite3StdType":                               {},
	"sqlite3StdTypeAffinity":                       {},
	"sqlite3StdTypeLen":                            {},
	"sqlite3StdTypeMap":                            {},
	"sqlite3StmtCurrentTime":                       {},
	"sqlite3StorageColumnToTable":                  {},
	"sqlite3StrAccumEnlarge":                       {},
	"sqlite3StrAccumFinish":                        {},
	"sqlite3StrAccumInit":                          {},
	"sqlite3StrAccumSetError":                      {},
	"sqlite3StrBINARY":                             {},
	"sqlite3StrICmp":                               {},
	"sqlite3StrIHash":                              {},
	"sqlite3Strlen30":                              {},
	"sqlite3SubInt64":                              {},
	"sqlite3SubselectError":                        {},
	"sqlite3SystemError":                           {},
	"sqlite3TableAffinity":                         {},
	"sqlite3TableColumnAffinity":                   {},
	"sqlite3TableColumnToIndex":                    {},
	"sqlite3TableColumnToStorage":                  {},
	"sqlite3TableLock":                             {},
	"sqlite3TempInMemory":                          {},
	"sqlite3ThreadCreate":                          {},
	"sqlite3ThreadJoin":                            {},
	"sqlite3TokenInit":                             {},
	"sqlite3TransferBindings":                      {},
	"sqlite3TreeTrace":                             {},
	"sqlite3TriggerColmask":                        {},
	"sqlite3TriggerDeleteStep":                     {},
	"sqlite3TriggerInsertStep":                     {},
	"sqlite3TriggerList":                           {},
	"sqlite3TriggerSelectStep":                     {},
	"sqlite3TriggerStepSrc":                        {},
	"sqlite3TriggerUpdateStep":                     {},
	"sqlite3TriggersExist":                         {},
	"sqlite3TwoPartName":                           {},
	"sqlite3UniqueConstraint":                      {},
	"sqlite3UnlinkAndDeleteIndex":                  {},
	"sqlite3UnlinkAndDeleteTable":                  {},
	"sqlite3UnlinkAndDeleteTrigger":                {},
	"sqlite3Update":                                {},
	"sqlite3UpperToLower":                          {},
	"sqlite3UpsertAnalyzeTarget":                   {},
	"sqlite3UpsertDelete":                          {},
	"sqlite3UpsertDoUpdate":                        {},
	"sqlite3UpsertDup":                             {},
	"sqlite3UpsertNew":                             {},
	"sqlite3UpsertNextIsIPK":                       {},
	"sqlite3UpsertOfIndex":                         {},
	"sqlite3Utf16ByteLen":                          {},
	"sqlite3Utf16to8":                              {},
	"sqlite3Utf8CharLen":                           {},
	"sqlite3Utf8Read":                              {},
	"sqlite3VListAdd":                              {},
	"sqlite3VListNameToNum":                        {},
	"sqlite3VListNumToName":                        {},
	"sqlite3VMPrintf":                              {},
	"sqlite3Vacuum":                                {},
	"sqlite3ValueApplyAffinity":                    {},
	"sqlite3ValueBytes":                            {},
	"sqlite3ValueFree":                             {},
	"sqlite3ValueFromExpr":                         {},
	"sqlite3ValueNew":                              {},
	"sqlite3ValueSetNull":                          {},
	"sqlite3ValueSetStr":                           {},
	"sqlite3ValueText":                             {},
	"sqlite3VarintLen":                             {},
	"sqlite3VdbeAddFunctionCall":                   {},
	"sqlite3VdbeAddOp0":                            {},
	"sqlite3VdbeAddOp1":                            {},
	"sqlite3VdbeAddOp2":                            {},
	"sqlite3VdbeAddOp3":                            {},
	"sqlite3VdbeAddOp4":                            {},
	"sqlite3VdbeAddOp4Dup8":                        {},
	"sqlite3VdbeAddOp4Int":                         {},
	"sqlite3VdbeAddOpList":                         {},
	"sqlite3VdbeAddParseSchemaOp":                  {},
	"sqlite3VdbeAllocUnpackedRecord":               {},
	"sqlite3VdbeAppendP4":                          {},
	"sqlite3VdbeBooleanValue":                      {},
	"sqlite3VdbeChangeEncoding":                    {},
	"sqlite3VdbeChangeOpcode":                      {},
	"sqlite3VdbeChangeP1":                          {},
	"sqlite3VdbeChangeP2":                          {},
	"sqlite3VdbeChangeP3":                          {},
	"sqlite3VdbeChangeP4":                          {},
	"sqlite3VdbeChangeP5":                          {},
	"sqlite3VdbeChangeToNoop":                      {},
	"sqlite3VdbeCheckFk":                           {},
	"sqlite3VdbeCloseStatement":                    {},
	"sqlite3VdbeCountChanges":                      {},
	"sqlite3VdbeCreate":                            {},
	"sqlite3VdbeCurrentAddr":                       {},
	"sqlite3VdbeCursorRestore":                     {},
	"sqlite3VdbeDb":                                {},
	"sqlite3VdbeDelete":                            {},
	"sqlite3VdbeDeleteAuxData":                     {},
	"sqlite3VdbeDeletePriorOpcode":                 {},
	"sqlite3VdbeDisplayP4":                         {},
	"sqlite3VdbeEndCoroutine":                      {},
	"sqlite3VdbeEnter":                             {},
	"sqlite3VdbeError":                             {},
	"sqlite3VdbeExec":                              {},
	"sqlite3VdbeExpandSql":                         {},
	"sqlite3VdbeExplain":                           {},
	"sqlite3VdbeExplainParent":                     {},
	"sqlite3VdbeExplainPop":                        {},
	"sqlite3VdbeFinalize":                          {},
	"sqlite3VdbeFindCompare":                       {},
	"sqlite3VdbeFinishMoveto":                      {},
	"sqlite3VdbeFrameDelete":                       {},
	"sqlite3VdbeFrameMemDel":                       {},
	"sqlite3VdbeFrameRestore":                      {},
	"sqlite3VdbeFreeCursor":                        {},
	"sqlite3VdbeFreeCursorNN":                      {},
	"sqlite3VdbeGetBoundValue":                     {},
	"sqlite3VdbeGetOp":                             {},
	"sqlite3VdbeGoto":                              {},
	"sqlite3VdbeHalt":                              {},
	"sqlite3VdbeHandleMovedCursor":                 {},
	"sqlite3VdbeHasSubProgram":                     {},
	"sqlite3VdbeIdxKeyCompare":                     {},
	"sqlite3VdbeIdxRowid":                          {},
	"sqlite3VdbeIntValue":                          {},
	"sqlite3VdbeIntegerAffinity":                   {},
	"sqlite3VdbeJumpHere":                          {},
	"sqlite3VdbeJumpHereOrPopInst":                 {},
	"sqlite3VdbeLeave":                             {},
	"sqlite3VdbeLinkSubProgram":                    {},
	"sqlite3VdbeList":                              {},
	"sqlite3VdbeLoadString":                        {},
	"sqlite3VdbeMakeLabel":                         {},
	"sqlite3VdbeMakeReady":                         {},
	"sqlite3VdbeMemAggValue":                       {},
	"sqlite3VdbeMemCast":                           {},
	"sqlite3VdbeMemClearAndResize":                 {},
	"sqlite3VdbeMemCopy":                           {},
	"sqlite3VdbeMemExpandBlob":                     {},
	"sqlite3VdbeMemFinalize":                       {},
	"sqlite3VdbeMemFromBtree":                      {},
	"sqlite3VdbeMemFromBtreeZeroOffset":            {},
	"sqlite3VdbeMemGrow":                           {},
	"sqlite3VdbeMemHandleBom":                      {},
	"sqlite3VdbeMemInit":                           {},
	"sqlite3VdbeMemIntegerify":                     {},
	"sqlite3VdbeMemMakeWriteable":                  {},
	"sqlite3VdbeMemMove":                           {},
	"sqlite3VdbeMemNulTerminate":                   {},
	"sqlite3VdbeMemNumerify":                       {},
	"sqlite3VdbeMemRealify":                        {},
	"sqlite3VdbeMemRelease":                        {},
	"sqlite3VdbeMemReleaseMalloc":                  {},
	"sqlite3VdbeMemSetDouble":                      {},
	"sqlite3VdbeMemSetInt64":                       {},
	"sqlite3VdbeMemSetNull":                        {},
	"sqlite3VdbeMemSetPointer":                     {},
	"sqlite3VdbeMemSetRowSet":                      {},
	"sqlite3VdbeMemSetStr":                         {},
	"sqlite3VdbeMemSetZeroBlob":                    {},
	"sqlite3VdbeMemShallowCopy":                    {},
	"sqlite3VdbeMemStringify":                      {},
	"sqlite3VdbeMemTooBig":                         {},
	"sqlite3VdbeMemTranslate":                      {},
	"sqlite3VdbeMultiLoad":                         {},
	"sqlite3VdbeNextOpcode":                        {},
	"sqlite3VdbeOneByteSerialTypeLen":              {},
	"sqlite3VdbeParameterIndex":                    {},
	"sqlite3VdbeParser":                            {},
	"sqlite3VdbePreUpdateHook":                     {},
	"sqlite3VdbePrepareFlags":                      {},
	"sqlite3VdbeRealValue":                         {},
	"sqlite3VdbeRecordCompare":                     {},
	"sqlite3VdbeRecordCompareWithSkip":             {},
	"sqlite3VdbeRecordUnpack":                      {},
	"sqlite3VdbeReset":                             {},
	"sqlite3VdbeResetStepResult":                   {},
	"sqlite3VdbeResolveLabel":                      {},
	"sqlite3VdbeReusable":                          {},
	"sqlite3VdbeRewind":                            {},
	"sqlite3VdbeRunOnlyOnce":                       {},
	"sqlite3VdbeSerialGet":                         {},
	"sqlite3VdbeSerialTypeLen":                     {},
	"sqlite3VdbeSetChanges":                        {},
	"sqlite3VdbeSetColName":                        {},
	"sqlite3VdbeSetNumCols":                        {},
	"sqlite3VdbeSetP4KeyInfo":                      {},
	"sqlite3VdbeSetSql":                            {},
	"sqlite3VdbeSetVarmask":                        {},
	"sqlite3VdbeSorterClose":                       {},
	"sqlite3VdbeSorterCompare":                     {},
	"sqlite3VdbeSorterInit":                        {},
	"sqlite3VdbeSorterNext":                        {},
	"sqlite3VdbeSorterReset":                       {},
	"sqlite3VdbeSorterRewind":                      {},
	"sqlite3VdbeSorterRowkey":                      {},
	"sqlite3VdbeSorterWrite":                       {},
	"sqlite3VdbeSwap":                              {},
	"sqlite3VdbeTakeOpArray":                       {},
	"sqlite3VdbeTransferError":                     {},
	"sqlite3VdbeUsesBtree":                         {},
	"sqlite3VectorErrorMsg":                        {},
	"sqlite3VectorFieldSubexpr":                    {},
	"sqlite3ViewGetColumnNames":                    {},
	"sqlite3VtabArgExtend":                         {},
	"sqlite3VtabArgInit":                           {},
	"sqlite3VtabBegin":                             {},
	"sqlite3VtabBeginParse":                        {},
	"sqlite3VtabCallConnect":                       {},
	"sqlite3VtabCallCreate":                        {},
	"sqlite3VtabCallDestroy":                       {},
	"sqlite3VtabClear":                             {},
	"sqlite3VtabCommit":                            {},
	"sqlite3VtabCreateModule":                      {},
	"sqlite3VtabDisconnect":                        {},
	"sqlite3VtabEponymousTableClear":               {},
	"sqlite3VtabEponymousTableInit":                {},
	"sqlite3VtabFinishParse":                       {},
	"sqlite3VtabImportErrmsg":                      {},
	"sqlite3VtabLock":                              {},
	"sqlite3VtabMakeWritable":                      {},
	"sqlite3VtabModuleUnref":                       {},
	"sqlite3VtabOverloadFunction":                  {},
	"sqlite3VtabRollback":                          {},
	"sqlite3VtabSavepoint":                         {},
	"sqlite3VtabSync":                              {},
	"sqlite3VtabUnlock":                            {},
	"sqlite3VtabUnlockList":                        {},
	"sqlite3WalBeginReadTransaction":               {},
	"sqlite3WalBeginWriteTransaction":              {},
	"sqlite3WalCallback":                           {},
	"sqlite3WalCheckpoint":                         {},
	"sqlite3WalClose":                              {},
	"sqlite3WalDbsize":                             {},
	"sqlite3WalDefaultHook":                        {},
	"sqlite3WalEndReadTransaction":                 {},
	"sqlite3WalEndWriteTransaction":                {},
	"sqlite3WalExclusiveMode":                      {},
	"sqlite3WalFile":                               {},
	"sqlite3WalFindFrame":                          {},
	"sqlite3WalFrames":                             {},
	"sqlite3WalHeapMemory":                         {},
	"sqlite3WalLimit":                              {},
	"sqlite3WalOpen":                               {},
	"sqlite3WalReadFrame":                          {},
	"sqlite3WalSavepoint":                          {},
	"sqlite3WalSavepointUndo":                      {},
	"sqlite3WalSnapshotCheck":                      {},
	"sqlite3WalSnapshotGet":                        {},
	"sqlite3WalSnapshotOpen":                       {},
	"sqlite3WalSnapshotRecover":                    {},
	"sqlite3WalSnapshotUnlock":                     {},
	"sqlite3WalUndo":                               {},
	"sqlite3WalkExpr":                              {},
	"sqlite3WalkExprList":                          {},
	"sqlite3WalkSelect":                            {},
	"sqlite3WalkSelectExpr":                        {},
	"sqlite3WalkSelectFrom":                        {},
	"sqlite3WalkWinDefnDummyCallback":              {},
	"sqlite3WalkerDepthDecrease":                   {},
	"sqlite3WalkerDepthIncrease":                   {},
	"sqlite3WhereAddLimit":                         {},
	"sqlite3WhereBegin":                            {},
	"sqlite3WhereBreakLabel":                       {},
	"sqlite3WhereClauseClear":                      {},
	"sqlite3WhereClauseInit":                       {},
	"sqlite3WhereCodeOneLoopStart":                 {},
	"sqlite3WhereContinueLabel":                    {},
	"sqlite3WhereEnd":                              {},
	"sqlite3WhereExplainBloomFilter":               {},
	"sqlite3WhereExplainOneScan":                   {},
	"sqlite3WhereExprAnalyze":                      {},
	"sqlite3WhereExprListUsage":                    {},
	"sqlite3WhereExprUsage":                        {},
	"sqlite3WhereExprUsageNN":                      {},
	"sqlite3WhereFindTerm":                         {},
	"sqlite3WhereGetMask":                          {},
	"sqlite3WhereIsDistinct":                       {},
	"sqlite3WhereIsOrdered":                        {},
	"sqlite3WhereIsSorted":                         {},
	"sqlite3WhereMalloc":                           {},
	"sqlite3WhereMinMaxOptEarlyOut":                {},
	"sqlite3WhereOkOnePass":                        {},
	"sqlite3WhereOrderByLimitOptLabel":             {},
	"sqlite3WhereOutputRowCount":                   {},
	"sqlite3WhereRealloc":                          {},
	"sqlite3WhereRightJoinLoop":                    {},
	"sqlite3WhereSplit":                            {},
	"sqlite3WhereTabFuncArgs":                      {},
	"sqlite3WhereTrace":                            {},
	"sqlite3WhereUsesDeferredSeek":                 {},
	"sqlite3WindowAlloc":                           {},
	"sqlite3WindowAssemble":                        {},
	"sqlite3WindowAttach":                          {},
	"sqlite3WindowChain":                           {},
	"sqlite3WindowCodeInit":                        {},
	"sqlite3WindowCodeStep":                        {},
	"sqlite3WindowCompare":                         {},
	"sqlite3WindowDelete":                          {},
	"sqlite3WindowDup":                             {},
	"sqlite3WindowFunctions":                       {},
	"sqlite3WindowLink":                            {},
	"sqlite3WindowListDelete":                      {},
	"sqlite3WindowListDup":                         {},
	"sqlite3WindowRewrite":                         {},
	"sqlite3WindowUnlinkFromSelect":                {},
	"sqlite3WindowUpdate":                          {},
	"sqlite3WithAdd":                               {},
	"sqlite3WithDelete":                            {},
	"sqlite3WithDup":                               {},
	"sqlite3WithPush":                              {},
	"sqlite3WritableSchema":                        {},
	"sqlite3_aggregate_context":                    {},
	"sqlite3_aggregate_count":                      {},
	"sqlite3_auto_extension":                       {},
	"sqlite3_autovacuum_pages":                     {},
	"sqlite3_backup_finish":                        {},
	"sqlite3_backup_init":                          {},
	"sqlite3_backup_pagecount":                     {},
	"sqlite3_backup_remaining":                     {},
	"sqlite3_backup_step":                          {},
	"sqlite3_bind_blob":                            {},
	"sqlite3_bind_blob64":                          {},
	"sqlite3_bind_double":                          {},
	"sqlite3_bind_int":                             {},
	"sqlite3_bind_int64":                           {},
	"sqlite3_bind_null":                            {},
	"sqlite3_bind_parameter_count":                 {},
	"sqlite3_bind_parameter_index":                 {},
	"sqlite3_bind_parameter_name":                  {},
	"sqlite3_bind_pointer":                         {},
	"sqlite3_bind_text":                            {},
	"sqlite3_bind_text16":                          {},
	"sqlite3_bind_text64":                          {},
	"sqlite3_bind_value":                           {},
	"sqlite3_bind_zeroblob":                        {},
	"sqlite3_bind_zeroblob64":                      {},
	"sqlite3_blob_bytes":                           {},
	"sqlite3_blob_close":                           {},
	"sqlite3_blob_open":                            {},
	"sqlite3_blob_read":                            {},
	"sqlite3_blob_reopen":                          {},
	"sqlite3_blob_write":                           {},
	"sqlite3_busy_handler":                         {},
	"sqlite3_busy_timeout":                         {},
	"sqlite3_cancel_auto_extension":                {},
	"sqlite3_changes":                              {},
	"sqlite3_changes64":                            {},
	"sqlite3_clear_bindings":                       {},
	"sqlite3_close":                                {},
	"sqlite3_close_v2":                             {},
	"sqlite3_collation_needed":                     {},
	"sqlite3_collation_needed16":                   {},
	"sqlite3_column_blob":                          {},
	"sqlite3_column_bytes":                         {},
	"sqlite3_column_bytes16":                       {},
	"sqlite3_column_count":                         {},
	"sqlite3_column_database_name":                 {},
	"sqlite3_column_database_name16":               {},
	"sqlite3_column_decltype":                      {},
	"sqlite3_column_decltype16":                    {},
	"sqlite3_column_double":                        {},
	"sqlite3_column_int":                           {},
	"sqlite3_column_int64":                         {},
	"sqlite3_column_name":                          {},
	"sqlite3_column_name16":                        {},
	"sqlite3_column_origin_name":                   {},
	"sqlite3_column_origin_name16":                 {},
	"sqlite3_column_table_name":                    {},
	"sqlite3_column_table_name16":                  {},
	"sqlite3_column_text":                          {},
	"sqlite3_column_text16":                        {},
	"sqlite3_column_type":                          {},
	"sqlite3_column_value":                         {},
	"sqlite3_commit_hook":                          {},
	"sqlite3_compileoption_get":                    {},
	"sqlite3_compileoption_used":                   {},
	"sqlite3_complete":                             {},
	"sqlite3_complete16":                           {},
	"sqlite3_config":                               {},
	"sqlite3_context_db_handle":                    {},
	"sqlite3_create_collation":                     {},
	"sqlite3_create_collation16":                   {},
	"sqlite3_create_collation_v2":                  {},
	"sqlite3_create_filename":                      {},
	"sqlite3_create_function":                      {},
	"sqlite3_create_function16":                    {},
	"sqlite3_create_function_v2":                   {},
	"sqlite3_create_module":                        {},
	"sqlite3_create_module_v2":                     {},
	"sqlite3_create_window_function":               {},
	"sqlite3_data_count":                           {},
	"sqlite3_data_directory":                       {},
	"sqlite3_database_file_object":                 {},
	"sqlite3_db_cacheflush":                        {},
	"sqlite3_db_config":                            {},
	"sqlite3_db_filename":                          {},
	"sqlite3_db_handle":                            {},
	"sqlite3_db_mutex":                             {},
	"sqlite3_db_name":                              {},
	"sqlite3_db_readonly":                          {},
	"sqlite3_db_release_memory":                    {},
	"sqlite3_db_status":                            {},
	"sqlite3_declare_vtab":                         {},
	"sqlite3_deserialize":                          {},
	"sqlite3_drop_modules":                         {},
	"sqlite3_enable_load_extension":                {},
	"sqlite3_enable_shared_cache":                  {},
	"sqlite3_errcode":                              {},
	"sqlite3_errmsg":                               {},
	"sqlite3_errmsg16":                             {},
	"sqlite3_error_offset":                         {},
	"sqlite3_errstr":                               {},
	"sqlite3_exec":                                 {},
	"sqlite3_expanded_sql":                         {},
	"sqlite3_expired":                              {},
	"sqlite3_extended_errcode":                     {},
	"sqlite3_extended_result_codes":                {},
	"sqlite3_file_control":                         {},
	"sqlite3_filename_database":                    {},
	"sqlite3_filename_journal":                     {},
	"sqlite3_filename_wal":                         {},
	"sqlite3_finalize":                             {},
	"sqlite3_free":                                 {},
	"sqlite3_free_filename":                        {},
	"sqlite3_free_table":                           {},
	"sqlite3_get_autocommit":                       {},
	"sqlite3_get_auxdata":                          {},
	"sqlite3_get_table":                            {},
	"sqlite3_global_recover":                       {},
	"sqlite3_hard_heap_limit64":                    {},
	"sqlite3_initialize":                           {},
	"sqlite3_interrupt":                            {},
	"sqlite3_keyword_check":                        {},
	"sqlite3_keyword_count":                        {},
	"sqlite3_keyword_name":                         {},
	"sqlite3_last_insert_rowid":                    {},
	"sqlite3_libversion":                           {},
	"sqlite3_libversion_number":                    {},
	"sqlite3_limit":                                {},
	"sqlite3_load_extension":                       {},
	"sqlite3_log":                                  {},
	"sqlite3_malloc":                               {},
	"sqlite3_malloc64":                             {},
	"sqlite3_memory_alarm":                         {},
	"sqlite3_memory_highwater":                     {},
	"sqlite3_memory_used":                          {},
	"sqlite3_mprintf":                              {},
	"sqlite3_msize":                                {},
	"sqlite3_mutex_alloc":                          {},
	"sqlite3_mutex_enter":                          {},
	"sqlite3_mutex_free":                           {},
	"sqlite3_mutex_leave":                          {},
	"sqlite3_mutex_try":                            {},
	"sqlite3_next_stmt":                            {},
	"sqlite3_open":                                 {},
	"sqlite3_open16":                               {},
	"sqlite3_open_v2":                              {},
	"sqlite3_os_end":                               {},
	"sqlite3_os_init":                              {},
	"sqlite3_overload_function":                    {},
	"sqlite3_prepare":                              {},
	"sqlite3_prepare16":                            {},
	"sqlite3_prepare16_v2":                         {},
	"sqlite3_prepare16_v3":                         {},
	"sqlite3_prepare_v2":                           {},
	"sqlite3_prepare_v3":                           {},
	"sqlite3_preupdate_blobwrite":                  {},
	"sqlite3_preupdate_count":                      {},
	"sqlite3_preupdate_depth":                      {},
	"sqlite3_preupdate_hook":                       {},
	"sqlite3_preupdate_new":                        {},
	"sqlite3_preupdate_old":                        {},
	"sqlite3_profile":                              {},
	"sqlite3_progress_handler":                     {},
	"sqlite3_randomness":                           {},
	"sqlite3_realloc":                              {},
	"sqlite3_realloc64":                            {},
	"sqlite3_release_memory":                       {},
	"sqlite3_reset":                                {},
	"sqlite3_reset_auto_extension":                 {},
	"sqlite3_result_blob":                          {},
	"sqlite3_result_blob64":                        {},
	"sqlite3_result_double":                        {},
	"sqlite3_result_error":                         {},
	"sqlite3_result_error16":                       {},
	"sqlite3_result_error_code":                    {},
	"sqlite3_result_error_nomem":                   {},
	"sqlite3_result_error_toobig":                  {},
	"sqlite3_result_int":                           {},
	"sqlite3_result_int64":                         {},
	"sqlite3_result_null":                          {},
	"sqlite3_result_pointer":                       {},
	"sqlite3_result_subtype":                       {},
	"sqlite3_result_text":                          {},
	"sqlite3_result_text16":                        {},
	"sqlite3_result_text16be":                      {},
	"sqlite3_result_text16le":                      {},
	"sqlite3_result_text64":                        {},
	"sqlite3_result_value":                         {},
	"sqlite3_result_zeroblob":                      {},
	"sqlite3_result_zeroblob64":                    {},
	"sqlite3_rollback_hook":                        {},
	"sqlite3_rtree_geometry_callback":              {},
	"sqlite3_rtree_query_callback":                 {},
	"sqlite3_serialize":                            {},
	"sqlite3_set_authorizer":                       {},
	"sqlite3_set_auxdata":                          {},
	"sqlite3_set_last_insert_rowid":                {},
	"sqlite3_shutdown":                             {},
	"sqlite3_sleep":                                {},
	"sqlite3_snapshot_cmp":                         {},
	"sqlite3_snapshot_free":                        {},
	"sqlite3_snapshot_get":                         {},
	"sqlite3_snapshot_open":                        {},
	"sqlite3_snapshot_recover":                     {},
	"sqlite3_snprintf":                             {},
	"sqlite3_soft_heap_limit":                      {},
	"sqlite3_soft_heap_limit64":                    {},
	"sqlite3_sourceid":                             {},
	"sqlite3_sql":                                  {},
	"sqlite3_status":                               {},
	"sqlite3_status64":                             {},
	"sqlite3_step":                                 {},
	"sqlite3_stmt_busy":                            {},
	"sqlite3_stmt_isexplain":                       {},
	"sqlite3_stmt_readonly":                        {},
	"sqlite3_stmt_status":                          {},
	"sqlite3_str_append":                           {},
	"sqlite3_str_appendall":                        {},
	"sqlite3_str_appendchar":                       {},
	"sqlite3_str_appendf":                          {},
	"sqlite3_str_errcode":                          {},
	"sqlite3_str_finish":                           {},
	"sqlite3_str_length":                           {},
	"sqlite3_str_new":                              {},
	"sqlite3_str_reset":                            {},
	"sqlite3_str_value":                            {},
	"sqlite3_str_vappendf":                         {},
	"sqlite3_strglob":                              {},
	"sqlite3_stricmp":                              {},
	"sqlite3_strlike":                              {},
	"sqlite3_strnicmp":                             {},
	"sqlite3_system_errno":                         {},
	"sqlite3_table_column_metadata":                {},
	"sqlite3_temp_directory":                       {},
	"sqlite3_test_control":                         {},
	"sqlite3_thread_cleanup":                       {},
	"sqlite3_threadsafe":                           {},
	"sqlite3_total_changes":                        {},
	"sqlite3_total_changes64":                      {},
	"sqlite3_trace":                                {},
	"sqlite3_trace_v2":                             {},
	"sqlite3_transfer_bindings":                    {},
	"sqlite3_txn_state":                            {},
	"sqlite3_unlock_notify":                        {},
	"sqlite3_update_hook":                          {},
	"sqlite3_uri_boolean":                          {},
	"sqlite3_uri_int64":                            {},
	"sqlite3_uri_key":                              {},
	"sqlite3_uri_parameter":                        {},
	"sqlite3_user_data":                            {},
	"sqlite3_value_blob":                           {},
	"sqlite3_value_bytes":                          {},
	"sqlite3_value_bytes16":                        {},
	"sqlite3_value_double":                         {},
	"sqlite3_value_dup":                            {},
	"sqlite3_value_free":                           {},
	"sqlite3_value_frombind":                       {},
	"sqlite3_value_int":                            {},
	"sqlite3_value_int64":                          {},
	"sqlite3_value_nochange":                       {},
	"sqlite3_value_numeric_type":                   {},
	"sqlite3_value_pointer":                        {},
	"sqlite3_value_subtype":                        {},
	"sqlite3_value_text":                           {},
	"sqlite3_value_text16":                         {},
	"sqlite3_value_text16be":                       {},
	"sqlite3_value_text16le":                       {},
	"sqlite3_value_type":                           {},
	"sqlite3_version":                              {},
	"sqlite3_vfs_find":                             {},
	"sqlite3_vfs_register":                         {},
	"sqlite3_vfs_unregister":                       {},
	"sqlite3_vmprintf":                             {},
	"sqlite3_vsnprintf":                            {},
	"sqlite3_vtab_collation":                       {},
	"sqlite3_vtab_config":                          {},
	"sqlite3_vtab_distinct":                        {},
	"sqlite3_vtab_in":                              {},
	"sqlite3_vtab_in_first":                        {},
	"sqlite3_vtab_in_next":                         {},
	"sqlite3_vtab_nochange":                        {},
	"sqlite3_vtab_on_conflict":                     {},
	"sqlite3_vtab_rhs_value":                       {},
	"sqlite3_wal_autocheckpoint":                   {},
	"sqlite3_wal_checkpoint":                       {},
	"sqlite3_wal_checkpoint_v2":                    {},
	"sqlite3_wal_hook":                             {},
	"sqlite3aEQb":                                  {},
	"sqlite3aGTb":                                  {},
	"sqlite3aLTb":                                  {},
	"sqlite3changegroup_add":                       {},
	"sqlite3changegroup_add_strm":                  {},
	"sqlite3changegroup_delete":                    {},
	"sqlite3changegroup_new":                       {},
	"sqlite3changegroup_output":                    {},
	"sqlite3changegroup_output_strm":               {},
	"sqlite3changeset_apply":                       {},
	"sqlite3changeset_apply_strm":                  {},
	"sqlite3changeset_apply_v2":                    {},
	"sqlite3changeset_apply_v2_strm":               {},
	"sqlite3changeset_concat":                      {},
	"sqlite3changeset_concat_strm":                 {},
	"sqlite3changeset_conflict":                    {},
	"sqlite3changeset_finalize":                    {},
	"sqlite3changeset_fk_conflicts":                {},
	"sqlite3changeset_invert":                      {},
	"sqlite3changeset_invert_strm":                 {},
	"sqlite3changeset_new":                         {},
	"sqlite3changeset_next":                        {},
	"sqlite3changeset_old":                         {},
	"sqlite3changeset_op":                          {},
	"sqlite3changeset_pk":                          {},
	"sqlite3changeset_start":                       {},
	"sqlite3changeset_start_strm":                  {},
	"sqlite3changeset_start_v2":                    {},
	"sqlite3changeset_start_v2_strm":               {},
	"sqlite3rbu_bp_progress":                       {},
	"sqlite3rbu_close":                             {},
	"sqlite3rbu_create_vfs":                        {},
	"sqlite3rbu_db":                                {},
	"sqlite3rbu_destroy_vfs":                       {},
	"sqlite3rbu_open":                              {},
	"sqlite3rbu_progress":                          {},
	"sqlite3rbu_savestate":                         {},
	"sqlite3rbu_state":                             {},
	"sqlite3rbu_step":                              {},
	"sqlite3rbu_temp_size":                         {},
	"sqlite3rbu_temp_size_limit":                   {},
	"sqlite3rbu_vacuum":                            {},
	"sqlite3rebaser_configure":                     {},
	"sqlite3rebaser_create":                        {},
	"sqlite3rebaser_delete":                        {},
	"sqlite3rebaser_rebase":                        {},
	"sqlite3rebaser_rebase_strm":                   {},
	"sqlite3session_attach":                        {},
	"sqlite3session_changeset":                     {},
	"sqlite3session_changeset_size":                {},
	"sqlite3session_changeset_strm":                {},
	"sqlite3session_config":                        {},
	"sqlite3session_create":                        {},
	"sqlite3session_delete":                        {},
	"sqlite3session_diff":                          {},
	"sqlite3session_enable":                        {},
	"sqlite3session_indirect":                      {},
	"sqlite3session_isempty":                       {},
	"sqlite3session_memory_used":                   {},
	"sqlite3session_object_config":                 {},
	"sqlite3session_patchset":                      {},
	"sqlite3session_patchset_strm":                 {},
	"sqlite3session_table_filter":                  {},
}
